/***************************************************************************
 *
 * PROJECT: The Dark Mod
 * $Revision: 2810 $
 * $Date: 2008-09-10 06:43:44 +0200 (Mi, 10 Sep 2008) $
 * $Author: greebo $
 *
 ***************************************************************************/

#include "../idlib/precompiled.h"
#pragma hdrstop

static bool init_version = FileVersionList("$Id: PlayerData.cpp 2810 2008-09-10 04:43:44Z greebo $", init_version);

#include "../DarkMod/DarkModGlobals.h"
#include "../DarkMod/PlayerData.h"

CDarkModPlayer::CDarkModPlayer()
{
	m_FrobEntity = NULL;
	m_FrobJoint = INVALID_JOINT;
	m_FrobID = 0;
	m_FrobEntityPrevious = NULL;
	m_LightgemValue = 0;

	// greebo: Initialise the frob trace contact material to avoid 
	// crashing during map save when nothing has been frobbed yet
	memset(&m_FrobTrace, 0, sizeof(trace_t));
}

void CDarkModPlayer::Save( idSaveGame *savefile ) const
{
	m_FrobEntity.Save(savefile);
	savefile->WriteJoint(m_FrobJoint);
	savefile->WriteInt(m_FrobID);
	savefile->WriteTrace(m_FrobTrace);
	m_FrobEntityPrevious.Save(savefile);
	savefile->WriteInt(m_LightgemValue);
	savefile->WriteFloat(m_fColVal);

	savefile->WriteInt(m_LightList.Num());
	for (int i = 0; i < m_LightList.Num(); i++)
	{
		m_LightList[i].Save(savefile);
	}
}

void CDarkModPlayer::Restore( idRestoreGame *savefile )
{
	m_FrobEntity.Restore(savefile);
	savefile->ReadJoint(m_FrobJoint);
	savefile->ReadInt(m_FrobID);
	savefile->ReadTrace(m_FrobTrace);
	m_FrobEntityPrevious.Restore(savefile);
	savefile->ReadInt(m_LightgemValue);
	savefile->ReadFloat(m_fColVal);
	
	int num;
	savefile->ReadInt(num);
	m_LightList.SetNum(num);
	for (int i = 0; i < num; i++)
	{
		m_LightList[i].Restore(savefile);
	}
}

int CDarkModPlayer::AddLight(idLight *light)
{
	if(light)
	{
		idEntityPtr<idLight> lightPtr;
		lightPtr = light;
		m_LightList.Append(lightPtr);
		DM_LOG(LC_FUNCTION, LT_DEBUG)LOGSTRING("%08lX [%s] %lu added to LightList\r", light, light->name.c_str(), m_LightList.Num());
	}

	return m_LightList.Num();
}

int CDarkModPlayer::RemoveLight(idLight *light)
{
	if(light)
	{
		for (int i = 0; i < m_LightList.Num(); i++)
		{
			if (m_LightList[i].GetEntity() == light) 
			{
				// Light found, remove it
				m_LightList.RemoveIndex(i);
				DM_LOG(LC_FUNCTION, LT_DEBUG)LOGSTRING("%08lX [%s] %lu removed from LightList\r", light, light->name.c_str(), m_LightList.Num());
				break;
			}
		}
	}

	return m_LightList.Num();
}
